"       NAME            Splitter
        AUTHOR          subbotin@rrg.msk.su
        FUNCTION        Movable bound between 2 views ala File Manager
        KEYWORDS	
ST-VERSIONS     VisualWorks 1.0, VisualWorks 2.0
        PREREQUISITES   none
        CONFLICTS       none
        VERSION         1.0
        DATE            26 April 1996
SUMMARY Splitter
The header pretty much says it all. The splitter is a composite that
holds exactly two components and allows shifting the bounds between
them in runtime. Unfortunately it isn't integrated with UI builder,
so if you want it on canvas, you'll have to create it programmatically
and insert as custom view.

WARNING: this fileout splitter-izes the inspector and browsers. If you
do not want it, go to file end and erase offending methods.
    A. Subbotin, with the help of A. Bakharevsky and A. Miasnikov
"!

'From VisualWorks(TM), Release 1.0 of 8 October 1992 on 25 April 1996 at 
3:29:17 pm'!

Controller subclass: #SplitterController
        instanceVariableNames: ''
        classVariableNames: ''
        poolDictionaries: ''
        category: 'Splitter'!
SplitterController comment:
'I am a controller for Splitter. I show the ''split'' cursor over splitter 
bar and alow user to drag the bar,
thus changing the layout of view''s components.'!


!SplitterController methodsFor: 'basic control sequence'!

controlInitialize
        "Show the <resize> cursor"

        view cursor show!

controlTerminate
        "Restore normal cursor."

        Cursor normal show! !

!SplitterController methodsFor: 'control defaults'!

controlActivity
        " Process the dragging of splitter bar "

        | points globalOrigin newPoint newRatio |
        self sensor redButtonPressed ifTrue: [
                        globalOrigin := self sensor globalOrigin.
                        points := view splitBounds moveTo: sensor 
globalCursorPoint.
                        points := view isVertical
                                                ifTrue: [Array with: points 
leftCenter with: points rightCenter]
                                                ifFalse: [Array with: 
points topCenter with: points bottomCenter].
                        newPoint := Screen default
                                                dragShape: points
                                                lineWidth: view 
splitterBarHalfWidth*2
                                                offset: self sensor 
globalCursorPoint+
                                                        (view isVertical 
ifTrue:[0@view splitterBarHalfWidth] ifFalse:[view splitterBarHalfWidth@0])
                                                gridPhase: globalOrigin
                                                gridSpacing: (view 
isVertical
                                                                ifTrue: 
[10000 @ 1]
                                                                ifFalse: [1 
@ 10000])
                                                boundedBy: (view 
validBarBounds translatedBy: globalOrigin)
                                                whileButton: 1
                                                isDown: true.
                        newRatio := newPoint - self sensor globalOrigin / 
self view bounds extent.
                        self view proportion: (self view isVertical
                                        ifTrue: [newRatio y]
                                        ifFalse: [newRatio x])]!

isControlActive
        " While cursor is over splitter bar "

        ^self isControlWanted!

isControlWanted
        " When cursor is over the splitter bar "

        ^self view splitBounds containsPoint: self sensor cursorPoint! !

CompositeView subclass: #Splitter
        instanceVariableNames: 'proportionHolder isVertical '
        classVariableNames: 'HorizontalSplitCursor VerticalSplitCursor '
        poolDictionaries: ''
        category: 'Splitter'!
Splitter comment:
'I am a composite part that holds exactly two components and allows to move 
the border between them
interactively (like Windows File Manager).

instance variables:
        proportionHolder <ValueHolder on:Number> the ratio {0..1} defines 
the current position of splitter bar
                                        0 means first component is 
completely invisible etc.
        isVertical <Boolean> Whether the split is left-right or top-bottom
class variables:
        HorizontalSplitCursor <Cursor>
        VerticalSplitCursor <Cursor>   The cursors that are shown over 
split bar
'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!
Splitter class
        instanceVariableNames: ''!

!Splitter class methodsFor: 'class initialize'!

initHorizontalCursor
        HorizontalSplitCursor := Cursor imageArray: #(
                2r0000000000000000
                2r0000001110000000
                2r0000001110000000
                2r0000001110000000
                2r0000001110000000
                2r0001001110001000
                2r0011001110001100
                2r0111111111111110
                2r0011001110001100
                2r0001001110001000
                2r0000001110000000
                2r0000001110000000
                2r0000001110000000
                2r0000001110000000
                2r0000000000000000
                2r0)
                        maskArray: #(
                2r0000011111000000
                2r0000011111000000
                2r0000011111000000
                2r0000011111000000
                2r0001111111011000
                2r0011111111011100
                2r1111111111111111
                2r1111111111111111
                2r1111111111111111
                2r0011111111011100
                2r0001111111011000
                2r0000011111000000
                2r0000011111000000
                2r0000011111000000
                2r0000011111000000
                2r0)
                hotSpot: 7@7 name: 'hSplit'!

initialize
        "Splitter initialize"

        self initHorizontalCursor; initVerticalCursor!

initVerticalCursor
        VerticalSplitCursor := Cursor imageArray: #(
                2r0000000000000000
                2r0000000100000000
                2r0000001110000000
                2r0000011111000000
                2r0000000100000000
                2r0000000100000000
                2r0111111111111110
                2r0111111111111110
                2r0111111111111110
                2r0000000100000000
                2r0000000100000000
                2r0000011111000000
                2r0000001110000000
                2r0000000100000000
                2r0000000000000000
                2r0)
                        maskArray: #(
                2r0000001110000000
                2r0000011111000000
                2r0000111111100000
                2r0000111111100000
                2r0000001110000000
                2r1111111111111111
                2r1111111111111111
                2r1111111111111111
                2r1111111111111111
                2r1111111111111111
                2r0000001110000000
                2r0000111111100000
                2r0000111111100000
                2r0000011111000000
                2r0000001110000000
                2r0)
                        hotSpot: 7@7 name: 'vSplit'.! !

!Splitter class methodsFor: 'instance creation'!

horizontalWithProportion: aNumber
        " Creates an empty horizontal splitter. Now you must add two 
components to it "
        " like you do to any composite part. That allows you to replace the 
composites "
        " with splitters in existing code "

        ^self new
                beHorizontal;
                proportion: aNumber;
                yourself!

left: firstVisualComponent right: secondVisualComponent proportion: aNumber
        " Create horizontal splitter with two components in it "

        ^self new
                beHorizontal;
                add: firstVisualComponent;
                add: secondVisualComponent;
                proportion: aNumber;
                yourself!

top: firstVisualComponent bottom: secondVisualComponent proportion: aNumber
        " Create vertical splitter with two components in it "

        ^self new
                beVertical;
                add: firstVisualComponent;
                add: secondVisualComponent;
                proportion: aNumber;
                yourself!

verticalWithProportion: aNumber
        " Creates an empty vertical splitter. Now you must add two 
components to it "
        " like you do to any composite part. That allows you to replace the 
composites "
        " with splitters in existing code "

        ^self new
                beVertical;
                proportion: aNumber;
                yourself! !

!Splitter class methodsFor: 'examples'!

example
        "Splitter example"

        ScheduledWindow new
                label: 'Splitter example';
                component: (self
                        top: (LookPreferences edgeDecorator on:TextView 
new)
                        bottom: (Button trigger model:false asValue; 
label:'Bottom')
                        proportion: 0.5);
                open! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!


!Splitter methodsFor: 'initialize-release'!

beHorizontal
        " Make splitter move left-right and stack components horizontally 
(like FileManager)"

        isVertical := false!

beVertical
        " Make splitter move up-down and stack components vertically "

        isVertical := true!

initialize
        super initialize.
        proportionHolder := 0.5 asValue.
        proportionHolder onChangeSend: #refresh to: self.
        isVertical := true.! !

!Splitter methodsFor: 'accessing'!

cursor
        " The cursor that will be shown over splitter bar "

        ^self isVertical
                ifTrue: [VerticalSplitCursor]
                ifFalse: [HorizontalSplitCursor]!

proportion
        ^self proportionHolder value!

proportion: aNumber
        " Set the new proportion. aNumber must be between 0 and 1 "

        self proportionHolder value: aNumber!

proportionHolder
        " If you wish to control the splitter programmatically, here you 
can retrieve the "
        " reference to ratio holder and set new ratios (0..1) "

        ^proportionHolder! !

!Splitter methodsFor: 'testing'!

isHorizontal
        ^self isVertical not!

isVertical
        ^isVertical! !

!Splitter methodsFor: 'displaying'!

displayOn:aGC
        " Draw the components and my splitter bar "

        super displayOn:aGC.
        aGC
                paint:self defaultColor;
                displayRectangle:self splitBounds.! !

!Splitter methodsFor: 'adding-removing'!

addWrapper: aWrapper
        " Make the layout of component dependent on my proportion "

        ^super addWrapper:(
                self boundedWrapperClass =aWrapper class
                        ifTrue:[ aWrapper layout:self createLayoutFrame. 
aWrapper]
                        ifFalse:[self boundedWrapperClass on: aWrapper in: 
self createLayoutFrame])! !

!Splitter methodsFor: 'controller accessing'!

defaultControllerClass
        ^SplitterController! !

!Splitter methodsFor: 'private'!

createLayoutFrame
        " Create a layout frame for either the first of the second 
component in splitter "

        self components size >= 2 ifTrue: [self error: 'Spitter must have 
exactly two components'].
        ^self components isEmpty        " If 1st component is being added "
                ifTrue: [self isVertical
                                ifTrue: [LayoutFrame leftFraction:0 
offset:0 rightFraction: 1 offset:0
                                                        topFraction: 0 
offset:0 bottomFraction: [self proportion] offset:self splitterBarHalfWidth 
negated]
                                ifFalse: [LayoutFrame leftFraction:0 
offset:0 rightFraction:[self proportion] offset:self splitterBarHalfWidth 
negated
                                                        topFraction:0 
offset:0 bottomFraction:1 offset:0]]
                ifFalse: [self isVertical
                                ifTrue: [LayoutFrame leftFraction:0 
offset:0 rightFraction:1 offset:0
                                                        topFraction: [self 
proportion] offset:self splitterBarHalfWidth bottomFraction:1 offset:0]
                                ifFalse: [LayoutFrame leftFraction: [self 
proportion] offset:self splitterBarHalfWidth rightFraction:1 offset:0
                                                        topFraction:0 
offset:0 bottomFraction:1 offset:0]].!

defaultColor
        " The color of splitter bar. Make it invisible on the background "
        " Other good ideas are (ColorValue black) and (SymbolicPaint 
foreground) "

        ^SymbolicPaint background!

refresh
        " If the window is open, force the relayout of its components "
        " Protect against incorrect ratio values "

        | window |
        self proportionHolder value<0 ifTrue:[self proportionHolder 
setValue:0].
        self proportionHolder value>1 ifTrue:[self proportionHolder 
setValue:1].
        ((window := self topComponent) notNil and:[window isOpen]) ifTrue: 
[window refresh]!

splitBounds
        " The bounds of split bar between two views "
        " When the cursor is within these bounds, drag results in splitter 
being moved "

        | bnd level |
        bnd := self bounds.
        level := bnd extent * self proportion asPoint + bnd origin.
        ^self isVertical
                ifTrue: [bnd left @ (level y -self splitterBarHalfWidth) 
corner: bnd right @ (level y + self splitterBarHalfWidth)]
                ifFalse: [(level x - self splitterBarHalfWidth)@bnd top 
corner: (level x + self splitterBarHalfWidth) @ bnd bottom]!

splitterBarHalfWidth
        " 1/2 of the width of splitter ribbon "

        ^1!

validBarBounds
        " The limits of splitter bar movement. We inset them a couple of 
pixels, "
        " because otherwise user will be able to drag the bar completely 
off-screen "

        ^self bounds insetBy: self splitterBarHalfWidth! !

Splitter initialize!



"==================== HERE BEGIN BROWSER MODIFICATIONS 
==================="!

!Inspector class methodsFor: 'view creation'!

buildScheduledView: anInspector
        | topWindow composite |
        topWindow := ScheduledWindow
                        model: anInspector
                        label: anInspector viewLabel
                        minimumSize: 180 @ 180.
" A. Subbotin. Replace the ordinary composite with splitter "
">      composite := DependentComposite new."
">"     composite :=Splitter horizontalWithProportion:0.4.
        topWindow component: composite.
        self view: anInspector in: (0@0 extent: 1@1) of: composite.
        topWindow icon: (Icon constantNamed: #inspector).
        ^ topWindow! !

!Browser class methodsFor: 'view creation'!

openProtocolBrowserOn: aBrowser withTextState: anArray
        "Create and schedule a browser on the category of messages selected 
by
        aBrowser.  anArray holds the initial text state."

        | topView splitterView lineHeightBlock topWindow |
        lineHeightBlock := self lineHeightBlock.
        topWindow := ScheduledWindow
                                model: aBrowser
                                label: aBrowser selectedClass name, ' 
Protocol Browser'
                                minimumSize: 400@200.
        topView := DependentComposite new.
        splitterView :=Splitter verticalWithProportion:0.4.
        splitterView add:topView.
        topWindow component: splitterView.
        self
                addProtocolViewTo: topView
                        in: (LayoutFrame new
                                leftOffset: 0;
                                rightFraction: 1;
                                topOffset: 0;
                                bottomOffset: lineHeightBlock) on: aBrowser 
readOnly: true;
                addSelectorViewTo: topView
                        in: (LayoutFrame new
                                leftOffset: 0;
                                rightFraction: 1;
                                topOffset: lineHeightBlock;
                                bottomFraction: 1) on: aBrowser readOnly: 
false;
                addTextViewTo: splitterView in: (0@0.4 corner: 1.0@1.0) on: 
aBrowser initialSelection: nil initialState: anArray.
        topWindow icon: (Icon constantNamed: #protocolBrowser).
        topWindow open! !

!Browser class methodsFor: 'view creation'!

openOn: aBrowser withTextState: anArray
        "Create and schedule a browser on an entire collection of organized
        classes.   anArray holds the initial text state."

        | topView divider topWindow buttonOffset splitterView |
        buttonOffset := self buttonOffset.
        topWindow := ScheduledWindow model: aBrowser label: 'System 
Browser'
                                        minimumSize: 400@250.
        divider := 1.
        topView := DependentComposite new.
        splitterView :=Splitter verticalWithProportion:0.35.
        splitterView add:topView.
        topWindow component: splitterView.
        self
                addCategoryViewTo: topView in: (0@0 extent: 0.25@divider) 
on: aBrowser readOnly: false;
                addClassViewTo: topView
                        in: (LayoutFrame new
                                leftFraction: 0.25;
                                topOffset: 0;
                                rightFraction: 0.50;
                                bottomFraction: divider offset: 
buttonOffset) on: aBrowser readOnly: false;
                addClassAndInstanceButtonViewTo: topView
                        in: (LayoutFrame new
                                leftFraction: 0.25;
                                topFraction: divider offset:  buttonOffset;
                                rightFraction: 0.5;
                                bottomFraction: divider)
                        on: aBrowser;
                addProtocolViewTo: topView in: (0.5@0 extent: 0.25@divider) 
on: aBrowser readOnly: false;
                addSelectorViewTo: topView in: (0.75@0 extent: 
0.25@divider) on: aBrowser readOnly: false;
                addTextViewTo: splitterView in: (0@0 corner: 1.0@1.0) on: 
aBrowser initialSelection: nil initialState: anArray.
        topWindow icon: (Icon constantNamed: #systemBrowser).
        topWindow openWithExtent: ((topWindow  minimumSize * 3 + topWindow 
maximumSize) // 4)! !

!MethodListBrowser class methodsFor: 'view creation'!

openListBrowserOn: aCollection label: labelString initialSelection: 
selector
        "Create and schedule a browser on the collection of messages 
aCollection.
        Label the topview labelString. If the collection is empty, warn the 
user."

        | topView topWindow aBrowser label splitterView |
        aCollection isEmpty ifTrue: [^DialogView warn: 'Nobody'].
        aBrowser := self new on: aCollection.
        label := (labelString isKindOf: LookupKey)
                                ifTrue: [labelString key]
                                ifFalse: [labelString asString].
        topWindow := ScheduledWindow
                                model: aBrowser
                                label: label
                                minimumSize: 300 @ 150.
        topView := DependentComposite new.
        splitterView :=Splitter verticalWithProportion:0.25.
        splitterView add:topView.
        topWindow component: splitterView.
        self
                addMethodViewTo: topView
                in: (0 @ 0 extent: 1.0 @ 1.0)
                on: aBrowser
                readOnly: false.
        self
                addTextViewTo: splitterView
                in: (0 @ 0.25 extent: 1 @ 0)
                on: aBrowser
                initialSelection: selector
                initialState: nil.
        topWindow icon: (Icon constantNamed: #methodBrowser).
        topWindow openWithExtent: 400@300! !

!Browser class methodsFor: 'view creation'!

openCategoryBrowserOn: aBrowser withTextState: anArray
        "Create and schedule a browser on the category of classes selected 
by
        aBrowser.  anArray holds the initial text state."

        | topView lineHeightBlock buttonOffset topWindow splitterView |
        lineHeightBlock := self lineHeightBlock.
        buttonOffset := self buttonOffset.
        topWindow := ScheduledWindow
                        model: aBrowser
                        label: (aBrowser category, ' Category Browser')
                        minimumSize: 400@250.
        topView := DependentComposite new.
        splitterView :=Splitter verticalWithProportion:0.4.
        splitterView add:topView.
        topWindow component: splitterView.
        self
                addCategoryViewTo: topView
                        in: (LayoutFrame new
                        leftOffset: 0;
                        topOffset: 0;
                        rightFraction: 0.3;
                        bottomOffset: lineHeightBlock) on: aBrowser 
readOnly: true;
                addClassViewTo: topView
                        in: (LayoutFrame new
                                leftOffset: 0;
                                topOffset: lineHeightBlock;
                                rightFraction: 0.3;
                                bottomFraction: 1 offset: buttonOffset) on: 
aBrowser readOnly: false;
                addClassAndInstanceButtonViewTo: topView
                        in: (LayoutFrame new
                                leftFraction: 0;
                                topFraction: 1 offset: buttonOffset;
                                rightFraction: 0.3;
                                bottomFraction: 1) on: aBrowser;
                addProtocolViewTo: topView in: (0.3@0 extent: 0.3@1.0) on: 
aBrowser readOnly: false;
                addSelectorViewTo: topView in: (0.6@0 extent: 0.4@1.0) on: 
aBrowser readOnly: false;
                addTextViewTo: splitterView in: (0@1.0 corner: 1.0@1.0) on: 
aBrowser initialSelection: nil initialState: anArray.
        topWindow icon: (Icon constantNamed: #categoryBrowser).
        topWindow openWithExtent: 400@300! !

!Browser class methodsFor: 'view creation'!

openClassBrowserOn: aBrowser withTextState: anArray
        "Create and schedule a browser on the class currently selected by 
aBrowser.
         anArray holds the initial text state."

        | topView topWindow lineHeightBlock buttonBottom splitterView |
        lineHeightBlock := self lineHeightBlock.
        buttonBottom := [lineHeightBlock value + LabeledBooleanView 
defaultHeight].
        topWindow := ScheduledWindow model: aBrowser
                                                label: aBrowser className,' 
Class Browser'
                                                minimumSize: 400@250.
        topView := DependentComposite new.
        splitterView :=Splitter verticalWithProportion:0.4.
        splitterView add:topView.
        topWindow component: splitterView.
        self
                addClassViewTo: topView
                        in: (LayoutFrame new
                                leftFraction: 0;
                                topOffset: 0;
                                rightFraction: 0.50;
                                bottomOffset: lineHeightBlock) on: aBrowser 
readOnly: true;
                addClassAndInstanceButtonViewTo: topView
                        in: (LayoutFrame new
                                leftFraction: 0;
                                topOffset: lineHeightBlock;
                                rightFraction: 0.5;
                                bottomOffset: buttonBottom)
                        on: aBrowser;
                addProtocolViewTo: topView in: (LayoutFrame new
                                leftOffset: 0;
                                topOffset: buttonBottom;
                                rightFraction: 0.5;
                                bottomFraction: 1) on: aBrowser readOnly: 
false;
                addSelectorViewTo: topView in: (LayoutFrame new
                        leftFraction: 0.5;
                        topOffset: 0;
                        rightFraction: 1;
                        bottomFraction: 1) on: aBrowser readOnly: false;
                addTextViewTo: splitterView in: (0@0.4 corner: 1.0@1.0) on: 
aBrowser initialSelection: nil initialState: anArray.
        topWindow icon: (Icon constantNamed: #classBrowser).
        topWindow openWithExtent: ((topWindow  minimumSize * 3 + topWindow 
maximumSize) // 4)! !

END--cut-here--cut-here


--
===========================
Andrey Subbotin
subbotin@rrg.msk.su





